import os, sys, toml, inspect


def load(path=None, upper=True, docker_env='DJANGO_ENV',
    production_env=('DJANGO_ENV', 'production'), module=None):
    """Use this method to load django settings from the pyproject.toml file.

    ### Params:

    - path - filepath of your toml file. If you don't specify it, django-pyproject will look in the folder that contains your project folder (with settings.py).
    - upper - whether to convert lowercase variable names to uppercase.
    - docker_env - django-pyproject will look for this env variable. If it finds the variable, django-pyproject will include settings from [tool.djange.docker].
    - production_env - django-pyproject will look for the env variable from production_env[0]. If it finds the variable and its contents match with production_env[1], django-pyproject will include settings from [tool.djange.production].
    - module - you can set custom module to which all settings will be uploaded.

    > Note: docker settings override normal and apps settings, production settings override all settings.
    """
    if not module:
        module = os.getenv('DJANGO_SETTINGS_MODULE')
    data, path = check(path)
    django_data = data['tool']['django']
    settings = { 'DEBUG': True }
    for key in django_data:
        if key not in ('production', 'docker', 'apps'):
            settings.update(convert(key, django_data[key], path, data, upper))
    apps = data.get('tool').get('django').get('apps')
    for app in apps:
        for key in apps[app]:
            settings.update(convert(key, apps[app][key], path, data, upper))
    if os.getenv(docker_env):
        django_data = data.get('tool').get('django').get('docker')
        if django_data:
            for key in django_data:
                settings.update(convert(key, django_data[key], path, data, upper))
    if os.getenv(production_env[0]) == production_env[1]:
        settings['DEBUG'] = False
        django_data = data.get('tool').get('django').get('production')
        if django_data:
            for key in django_data:
                settings.update(convert(key, django_data[key], path, data, upper))
    for key in settings:
        setattr(sys.modules[module], key, settings[key])
    
def check(path=None):
    if not path:
        filename = inspect.getouterframes(inspect.currentframe(), 2)[2].filename
        path = os.path.join(os.path.dirname(os.path.dirname(filename)), 'pyproject.toml')
    try:
        data = toml.load(path)
    except FileNotFoundError:
        raise FileNotFoundError(f"Path to pyproject.toml is not \
set or is incorrect, and default path didn't work. Broken filepath: {path}")
    except toml.TomlDecodeError:
        raise toml.TomlDecodeError('Your pyproject.toml is configured improperly.')
    try:
        django_data = data['tool']['django']
    except KeyError:
        raise KeyError("Your pyproject.toml doesn't have a [tool.django] section.")
    return data, path
    
def convert(key, value, path, data, upper=True):
    if upper:
        key = key.upper()
    if isinstance(value, dict):
        if 'env' in value:
            value = os.environ.get(value['env'], value.get('default'))
        elif 'path' in value:
            value = edit_path(value['path'], path)
        elif 'insert' in value:
            value = edit_var(key, value['insert'], data, value.get('pos'))
        else:
            for k in value: value[k] = convert(k, value[k], path, data, upper)
    return {key: value}
    
def edit_path(s, path):
    names = s.split('/')
    path = os.path.dirname(path)
    for i in range(1, len(names)):
        if names[i] == '..':
            path = os.path.dirname(path)
        else:
            path = os.path.join(path, names[i])
    return path
    
def edit_var(key, value, data, pos=None):
    result = data['tool']['django'].get(key, [])
    if pos: result.insert(pos, value)
    else: result.append(value)
    return result
    
def load_all(path=None):
    """Loads all data from the pyproject file as a dict.
    You can set custom path with the path parameter."""
    data, _ = check(path)
    return data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['djpp']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'django-pyproject',
    'version': '1.0.0',
    'description': 'A package for storing Django settings in pyproject.toml.',
    'long_description': '# django-pyproject\n\n## Description\n\nThis package allows you to store some/all of your settings in your pyproject.toml file (or any toml file in general).\n\n## Installation\n\nYou can add django-pyproject to your poetry project with:\n\n    poetry add django-pyproject\n\nOr through pip:\n\n    pip install django-pyproject\n\nNeither Django nor Poetry are **not** required to use this package.\n\n## Usage\n\nYou can use django-pyproject to import any settings specified in your pyproject file under [tool.django].\n\n### Settings file\n\nTo import django settings from your pyproject file, use this in your settings file:\n\n    from djpp import pyproject\n    pyproject.load()\n\nThis will work only if you have a standard django project structure.  \nIf your pyproject file is located somewhere else or has a different name, you can specify it:\n\n    pyproject.load(\'path-to-your-pyproject-file\')\n\n### PyProject file\n\nAll django settings in pyproject.toml file should be stored under [tool.django] key, like this:\n\n    [tool.django]\n    ALLOWED_HOSTS = []\n\nYou don\'t have to use uppercase letters for the variable names, django-pyproject will automatically convert them all. This does **not** work for dict key names:\n\n    [tool.django]\n    allowed_hosts = []\n\n    [tool.django.databases.default]\n    engine = \'django.db.backends.sqlite3\'\n    HOST = \'127.0.0.1\'\n    PORT = \'5432\'\n\nWill convert into:\n\n    ALLOWED_HOSTS = []\n    DATABASES = {\n        \'default\': {\n            \'engine\': \'django.db.backends.sqlite3\',\n            \'HOST\': \'127.0.0.1\',\n            \'PORT\': \'5432\',\n        }\n    }\n\nBut what to do about relative filepaths, that you had to construct with os.path?  \nYou can specify filepaths separating them with \'/\' in inline dict under key \'path\'.  \nUsing \'..\' will make django-pyproject go up a level.\nStarting with \'.\' will make a path relative:\n\n    base_dir = { path = "." }\n    project_dir = { path = "./your_project_folder" }\n    repo_dir = { path = "./.." }\n\nThis will have the same effect as the following code in settings.py:\n\n    import os\n    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))\n    PROJECT_DIR = os.path.dirname(os.path.abspath(__file__))\n    REPO_DIR = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))\n\n> This is assuming you have a standard django project structure.\n\nIf a value needs to be taken from an environment, use inline dict with key \'path\' and optional key \'default\':\n\n    email_host_password = { env = \'SECRET_PASSWORD\' }\n    secret_key = { env = \'SECRET_KEY\', default = \'hello\' }\n\n### Docker & Production\n\nIf some of you settings have an alternative value for when DEBUG is off, specify them in [tool.django.production]. They will override regular settings if DEBUG is off.\n\nBy default, django-pyproject applies production settings and sets DEBUG to False, if current evironment has a DJANGO_ENV variable, set to \'production\'.  \nYou can override it with your own key and value like this:\n\n    pyproject.load(production_env=(\'YOUR_KEY\', \'your_value\'))\n\nIf some of you settings have an alternative value for when the app is in container, specify them in [tool.django.docker]. They will override regular settings and will be overriden by production settings.\n\nBy default, django-pyproject applies docker settings, if current evironment has a DJANGO_ENV variable.  \nYou can override it with your own key like this:\n\n    pyproject.load(docker_env=\'YOUR_KEY\')\n\n### Apps\n\nYou can group settings that belong to an external app together for easier access.  \nTo do that, you can list them under [tool.django.apps.your_app].  \nYou can also modify variables like INSTALLED_APPS from here with \'insert\' key.\n\nHere\'s an example for corsheaders:\n\n    [tool.django.apps.cors]\n    CORS_ORIGIN_WHITELIST = [\'http://localhost:3000\',]\n    CORS_ALLOW_CREDENTIALS = true\n    CSRF_COOKIE_NAME = "XCSRF-Token"\n    INSTALLED_APPS = { insert = \'corsheaders\' }\n    MIDDLEWARE = { insert = \'corsheaders.middleware.CorsMiddleware\', pos = 3 }\n\nThis is similar to the following python code:\n\n    CORS_ORIGIN_WHITELIST = (\'http://localhost:3000\',)\n    CORS_ALLOW_CREDENTIALS = True\n    CSRF_COOKIE_NAME = "XCSRF-Token"\n    INSTALLED_APPS.append(\'corsheaders\')\n    MIDDLEWARE.insert(3, \'corsheaders.middleware.CorsMiddleware\')\n\n### Full import\n\nYou also can simply import pyproject file (or any toml file) contents as a dict with load_all().\n',
    'author': 'Ceterai',
    'author_email': 'ceterai@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
